package org.stepik.android.view.lesson.routing

import android.content.Intent
import org.stepic.droid.util.HtmlHelper
import org.stepik.android.domain.lesson.model.LessonDeepLinkData
import ru.nobird.android.view.base.ui.extension.getPathSegmentParameter

private const val PATH_SEGMENT_LESSON = "lesson"
private const val PATH_SEGMENT_STEP = "step"

private const val QUERY_PARAMETER_UNIT = "unit"
private const val QUERY_PARAMETER_DISCUSSION = "discussion"
private const val QUERY_PARAMETER_THREAD = "thread"

fun Intent.getLessonIdFromDeepLink(): Long? {
    val data = this.data ?: return null

    val path = data
        .getPathSegmentParameter(PATH_SEGMENT_LESSON)
        ?: return null

    return HtmlHelper.parseIdFromSlug(path)
}

fun Intent.getStepPositionFromDeepLink(): Int? {
    val data = this.data ?: return null

    val path = data
        .getPathSegmentParameter(PATH_SEGMENT_STEP)
        ?: return null

    return HtmlHelper.parseIdFromSlug(path)?.toInt()
}

fun Intent.getUnitIdFromDeepLink(): Long? {
    val data = this.data ?: return null

    val path = data
        .getQueryParameter(QUERY_PARAMETER_UNIT)
        ?: return null

    return HtmlHelper.parseIdFromSlug(path)
}

fun Intent.getDiscussionIdFromDeepLink(): Long? {
    val data = this.data ?: return null

    val path = data
        .getQueryParameter(QUERY_PARAMETER_DISCUSSION)
        ?: return null

    return HtmlHelper.parseIdFromSlug(path)
}

fun Intent.getThreadTypeFromDeepLink(): String? =
    this.data?.getQueryParameter(QUERY_PARAMETER_THREAD)

fun Intent.getLessonDeepLinkData(): LessonDeepLinkData? =
    getLessonIdFromDeepLink()
        ?.let { lessonId ->
            LessonDeepLinkData(
                lessonId = lessonId,
                stepPosition = getStepPositionFromDeepLink() ?: 1,
                unitId = getUnitIdFromDeepLink(),
                discussionId = getDiscussionIdFromDeepLink(), // todo: handle discussion id
                discussionThread = getThreadTypeFromDeepLink()
            )
        }